<?php
/* --------------------------------------------------------------
   PayPalLoaderHeaderExtender.inc.php 2020-02-20
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

class PayPalLoaderHeaderExtender extends PayPalLoaderHeaderExtender_parent
{
    public function proceed()
    {
        parent::proceed();
        
        $isEcsEnabled    = filter_var(gm_get_conf('GAMBIO_HUB_REMOTE_CONFIG_PAYPAL2HUB_ECS'), FILTER_VALIDATE_BOOLEAN);
        $isBannerEnabled = false;
        $bannerPositions = ['CARTBOTTOM', 'PRODUCT'];
        foreach ($bannerPositions as $bannerPosition) {
            $positionLayout = gm_get_conf('GAMBIO_HUB_REMOTE_CONFIG_PAYPAL2HUB_INSTALLMENTSBANNER_' . $bannerPosition . '_LAYOUT');
            $positionEnabled = $positionLayout !== null && $positionLayout !== 'none';
            $isBannerEnabled = $isBannerEnabled || $positionEnabled;
        }
        $isCheckoutPayment = strpos(gm_get_env_info('PHP_SELF'), 'checkout_payment.php') !== false;
        
        if ($isEcsEnabled === false && $isBannerEnabled === false && $isCheckoutPayment === false) {
            return;
        }
        
        /** @var \HubPayPalConfiguration $config */
        $config = MainFactory::create('HubPayPalConfiguration');
        $text   = MainFactory::create('LanguageTextManager', 'gambio_hub_paypal', $_SESSION['languages_id']);
        $payPalText = [
            'ecsNote'           => $text->get_text('ecs_note'),
            'paypalUnavailable' => $text->get_text('paypal_unavailable'),
            'errorContinue'     => $text->get_text('error_continue'),
        ];
        
        $paramsScript = '<script>';
        $paramsScript .= 'var payPalText = ' . json_encode($payPalText) . ';';
    
        /** @var \HubPayPalButtonSettings $buttonSettings */
        $buttonSettings = MainFactory::create('HubPayPalButtonSettings', $config);
        if (empty($_SESSION['customer_id'])
            && $_SESSION['customers_status']['customers_status_id'] === DEFAULT_CUSTOMERS_STATUS_ID_GUEST) {
            $checkoutUrl = xtc_href_link('shop.php',
                'do=PayPalHub/RedirectGuest',
                'SSL',
                false,
                false,
                false,
                false,
                true);
        } else {
            $checkoutUrl = xtc_href_link('checkout_shipping.php', '', 'SSL', false, false, false, false, true);
        }
        $buttonConfig = $buttonSettings->getButtonSettings();
        $buttonConfig['createPaymentUrl']     = xtc_href_link('shop.php', 'do=PayPalHub/CreatePayment&initiator=ecs', 'SSL', false, false, false, false, true);
        $buttonConfig['authorizedPaymentUrl'] = xtc_href_link('shop.php', 'do=PayPalHub/AuthorizedPayment&initiator=ecs', 'SSL', false, false, false, false, true);
        if(isset($_SESSION['gambio_hub_payments']['PayPal2Hub']) &&
           !isset($_SESSION['PayPal2Hub']['paymentID'], $_SESSION['PayPal2Hub']['payerID'])) {
            // ECM
            $buttonConfig['createPaymentUrl']     = xtc_href_link('shop.php', 'do=PayPalHub/CreatePayment', 'SSL', false, false, false, false, true);
            $buttonConfig['authorizedPaymentUrl'] = xtc_href_link('shop.php', 'do=PayPalHub/AuthorizedPayment', 'SSL', false, false, false, false, true);
        }
        $buttonConfig['checkoutUrl']          = $checkoutUrl;
        $buttonConfig['locale'] = $this->_getPayPalLocale();
        $buttonConfig['paymentApproved'] = isset($_SESSION['PayPal2Hub']['paymentID'], $_SESSION['PayPal2Hub']['payerID']);
    
        $buttonSettingsJson = json_encode($buttonConfig);
        $paramsScript .= 'var payPalButtonSettings = ' . $buttonSettingsJson . ';';
        
        if ($isBannerEnabled) {
            $bannerSettings = MainFactory::create('HubPayPalInstallmentsBannerSettings', $config);
            $bannerConfig = $bannerSettings->getInstallmentsBannerSettings();
            $bannerConfig['currency'] = $_SESSION['currency'];
            $bannerConfig['cartAmount'] = $_SESSION['cart']->show_total();
            $bannerSettingsJson = json_encode($bannerConfig);
            $paramsScript .= 'var payPalBannerSettings = ' . $bannerSettingsJson . ';';
        } else {
            $paramsScript .= 'var payPalBannerSettings = {};';
        }
        
        $paramsScript .= '</script>';
        
        if (!is_array($this->v_output_buffer)) {
            $this->v_output_buffer = [];
        }
        
        $this->v_output_buffer[] = $paramsScript . '<script src="GXModules/Gambio/Hub/Shop/Javascript/PayPalLoader.js" async></script>';
    }
    
    
    protected function _getPayPalLocale()
    {
        $paypalLocaleFactory = MainFactory::create('HubPayPalLocaleFactory');
        $locale = $paypalLocaleFactory->getLocaleByLanguageAndCountry($_SESSION['language_code'], $_SESSION['delivery_zone']);
        return $locale;
    }
}
