<?php
/**
 * ServiceProvider.php 2020-05-08
 * Gambio GmbH
 * http://www.gambio.de
 * Copyright (c) 2020 Gambio GmbH
 * Released under the GNU General Public License (Version 2)
 * [http://www.gnu.org/licenses/gpl-2.0.html]
 */

declare(strict_types=1);

namespace Gambio\Shop\GxCustomizer\Representation\Id;

use Gambio\Core\Event\EventListenerProvider;
use Gambio\Shop\GxCustomizer\Representation\Id\Factories\CustomizerPresentationIdFactory;
use Gambio\Shop\GxCustomizer\Representation\Id\Listener\OnPresentSellingUnitIdEventListener;
use Gambio\Shop\SellingUnit\Presentation\Events\OnPresentSellingUnitIdEvent;
use League\Container\Container;
use League\Container\ServiceProvider\AbstractServiceProvider;
use League\Container\ServiceProvider\BootableServiceProviderInterface;

/**
 * Class ServiceProvider
 * @package Gambio\Shop\GxCustomizer\Representation\Id
 * @property-read Container $container
 */
class ServiceProvider extends AbstractServiceProvider implements BootableServiceProviderInterface
{
    /**
     * @var string[]
     */
    protected $provides = [
        OnPresentSellingUnitIdEventListener::class
    ];
    
    
    public function register(): void
    {
        $this->container->share(CustomizerPresentationIdFactory::class);
        $this->container->share(OnPresentSellingUnitIdEventListener::class)
            ->addArgument(CustomizerPresentationIdFactory::class);
    }
    
    
    public function boot(): void
    {
        /** @var EventListenerProvider $listenerProvider */
        $listenerProvider = $this->container->get(EventListenerProvider::class);
        $listenerProvider->attachListener(OnPresentSellingUnitIdEvent::class, OnPresentSellingUnitIdEventListener::class);
    }
}